@@ -1,5 +1,6 @@
 package CGI::Wiki::Plugin::Bar;
-use base qw( CGI::Wiki::Plugin );
+use vars qw( @ISA );
+@ISA = qw( CGI::Wiki::Plugin );
 
 sub new {
     my $class = shift;
